@@ -9,13 +9,9 @@
 #' @param dat the data.frame with same columns as training data
 #' @param run.prediction Predict function of type: f(newdata)
 #' @param feature.name The column name of the feature for which to compute ALE
-#' @param grid.size The number of intervals
+#' @param grid.dt data.table with single column with grid values for the numerical feature
 #' @keywords internal
-calculate.ale.num <- function(dat, run.prediction, feature.name, grid.size) {
-  # from number of intervals to number of borders
-  n.borders <- grid.size + 1
-  # Handling duplicated grid values
-  grid.dt <- unique(get.grid(dat[, feature.name, with = FALSE], n.borders, type = "quantile"))
+calculate.ale.num <- function(dat, run.prediction, feature.name, grid.dt) {
   # Matching data instances to intervals
   interval.index <- findInterval(dat[[feature.name]], grid.dt[[1]], left.open = TRUE)
   # Data point in the left most interval should be in interval 1, not zero
@@ -30,6 +26,7 @@ calculate.ale.num <- function(dat, run.prediction, feature.name, grid.size) {
   prediction.deltas <- predictions.upper - predictions.lower
   deltas <- cbind(X.lower[, feature.name, with = FALSE], prediction.deltas, data.frame(".interval" = interval.index))
   y.hat.names <- as.data.table(setdiff(colnames(deltas), c(colnames(dat), ".interval")))
+  y.hat.names = unlist(y.hat.names)
   deltas <- data.table::melt(deltas,
     variable.name = ".class",
     value.name = ".yhat.diff", measure.vars = y.hat.names
@@ -68,19 +65,19 @@ calculate.ale.num <- function(dat, run.prediction, feature.name, grid.size) {
 #' @param dat the data.frame with same columns as training data
 #' @param run.prediction Predict function of type: f(newdata)
 #' @param feature.name The column names of the feature for which to compute ALE
-#' @param grid.size The number of cells
+#' @param grid.dt1 Data.table with single column with the grid value for feature 1
+#' @param grid.dt2 Data.table with single column with the grid value for feature 2
 #' @keywords internal
-calculate.ale.num.num <- function(dat, run.prediction, feature.name, grid.size) {
-  # Create grid for feature 1
-  grid.dt1 <- unique(get.grid(dat[, feature.name[1], with = FALSE], grid.size = grid.size[1] + 1, type = "quantile"))
-  colnames(grid.dt1) <- feature.name[1]
+calculate.ale.num.num <- function(dat, run.prediction, feature.name, grid.dt1, grid.dt2) {
+  # Remove data outside of boundaries
+  dat <- dat[(dat[[feature.name[1]]] <= max(grid.dt1[[1]])) &
+             (dat[[feature.name[1]]] >= min(grid.dt1[[1]])) &
+             (dat[[feature.name[2]]] <= max(grid.dt2[[1]])) &
+             (dat[[feature.name[2]]] >= min(grid.dt2[[1]])),]
   # Matching instances to the grid of feature 1
   interval.index1 <- findInterval(dat[[feature.name[1]]], grid.dt1[[1]], left.open = TRUE)
   # Data point in the left most interval should be in interval 1, not zero
   interval.index1[interval.index1 == 0] <- 1
-  ## Create grid for feature 2
-  grid.dt2 <- unique(get.grid(dat[, feature.name[2], with = FALSE], grid.size = grid.size[2] + 1, type = "quantile"))
-  colnames(grid.dt2) <- feature.name[2]
   # Matching instances to the grid of feature 2
   interval.index2 <- findInterval(dat[[feature.name[2]]], grid.dt2[[1]], left.open = TRUE)
   # Data point in the left most interval should be in interval 1, not zero
@@ -214,6 +211,7 @@ calculate.ale.num.num <- function(dat, run.prediction, feature.name, grid.size)
 #' @keywords internal
 calculate.ale.cat <- function(dat, run.prediction, feature.name) {
   x <- dat[, feature.name, with = FALSE][[1]]
+
   levels.original <- levels(droplevels(x))
   nlev <- nlevels(droplevels(x))
   # if ordered, than already use that
@@ -226,7 +224,7 @@ calculate.ale.cat <- function(dat, run.prediction, feature.name) {
   # The new order of the levels
   levels.ordered <- levels.original[level_order]
   # The feature with the levels in the new order
-  x.ordered <- order(level_order)[as.numeric(x)]
+  x.ordered <- order(level_order)[as.numeric(droplevels(x))]
   X.lower <- X.upper <- dat
 
   # We only want to increase/decrease the ones that are not already the minimum or maximum level
@@ -277,16 +275,20 @@ calculate.ale.cat <- function(dat, run.prediction, feature.name) {
 #' @param dat the data.frame with same columns as training data
 #' @param run.prediction Predict function of type: f(newdata)
 #' @param feature.name The column name of the features for which to compute ALE
-#' @param grid.size The number of intervals for the numerical feature
+#' @param grid.dt data.table with single column with grid values for the numerical feature
 #' @keywords internal
-calculate.ale.num.cat <- function(dat, run.prediction, feature.name, grid.size) {
+calculate.ale.num.cat <- function(dat, run.prediction, feature.name, grid.dt) {
 
   # Figure out which feature is numeric and which categeorical
   x.num.index <- ifelse(inherits(dat[, feature.name, with = FALSE][[1]], "numeric"), 1, 2)
-  x.cat.index <- setdiff(c(1, 2), x.num.index)
   x.num <- dat[, feature.name[x.num.index], with = FALSE][[1]]
-  x.cat <- dat[, feature.name[x.cat.index], with = FALSE][[1]]
+  # We can only compute ALE within min and max boundaries of given intervals
+  # This part is only relevat for user-defined intervals
+  dat <- dat[which((x.num >= min(grid.dt[[1]])) &
+                   (x.num <= max(grid.dt[[1]])))]
 
+  x.cat.index <- setdiff(c(1, 2), x.num.index)
+  x.cat <- dat[, feature.name[x.cat.index], with = FALSE][[1]]
   levels.original <- levels(x.cat)
   # if ordered, than already use that
   if (inherits(x.cat, "ordered")) {
@@ -302,13 +304,7 @@ calculate.ale.num.cat <- function(dat, run.prediction, feature.name, grid.size)
   # The rows for which the category can be increased
   row.ind.increase <- (1:nrow(dat))[x.cat.ordered < nlevels(x.cat)]
   row.ind.decrease <- (1:nrow(dat))[x.cat.ordered > 1]
-
-
-  ## Create ALE for increasing categorical feature
-  grid.dt <- unique(get.grid(dat[, feature.name[x.num.index], with = FALSE],
-    grid.size = grid.size[x.num.index] + 1, type = "quantile"
-  ))
-  colnames(grid.dt) <- feature.name[x.num.index]
+  
   interval.index <- findInterval(dat[[feature.name[x.num.index]]], grid.dt[[1]], left.open = TRUE)
   # Data point in the left most interval should be in interval 1, not zero
   interval.index[interval.index == 0] <- 1
@@ -476,7 +472,6 @@ calculate.ale.num.cat <- function(dat, run.prediction, feature.name, grid.size)
     ".level", ".num"
   )), with = FALSE]
   deltas$.type <- "ale"
-
   data.frame(deltas)
 }
 
